# Generated by Django 3.2.8 on 2021-11-28 04:41

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('phy_django', '0002_configuration'),
    ]

    operations = [
        migrations.AddField(
            model_name='configuration',
            name='user',
            field=phy_django.models.UserField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='用户'),
        ),
    ]
