# Generated by Django 3.2.8 on 2021-11-14 07:04

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='String',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True, verbose_name='数据库入库时间')),
                ('updated_datetime', models.DateTimeField(auto_now=True, verbose_name='数据库更新时间')),
                ('value', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.AddIndex(
            model_name='string',
            index=models.Index(fields=['value'], name='phy_django__value_1661eb_idx'),
        ),
    ]
