# Generated by Django 3.2.8 on 2021-11-28 04:50

from django.db import migrations
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        ('phy_django', '0003_configuration_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='configuration',
            name='value',
            field=phy_django.models.JSONField(default=dict, verbose_name='值'),
        ),
    ]
