# Generated by Django 3.2.8 on 2021-11-28 04:35

from django.db import migrations, models
import phy_django.models


class Migration(migrations.Migration):

    dependencies = [
        ('phy_django', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Configuration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True, verbose_name='数据库入库时间')),
                ('updated_datetime', models.DateTimeField(auto_now=True, verbose_name='数据库更新时间')),
                ('key', phy_django.models.ShortStringField(blank=True, default='', max_length=100, verbose_name='键')),
                ('value', models.JSONField(verbose_name='值')),
            ],
            options={
                'verbose_name': '配置',
                'verbose_name_plural': '配置',
                'abstract': False,
            },
        ),
    ]
